@@ -1,4 +1,4 @@
-# Copyright (C) 2017-2019 CS-SI. All Rights Reserved.
+# Copyright (C) 2017-2020 CS GROUP - France. All Rights Reserved.
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
 # This file is part of the Prewikka program.
@@ -60,7 +60,6 @@ class CronJob(object):
         self.id = id
         self.name = name
         self.user = user
-        self.schedule = schedule
         self.callback = func
         self.error = error
         self.ext_type = ext_type
@@ -73,6 +72,8 @@ class CronJob(object):
         self.set_schedule(schedule)
 
     def set_schedule(self, schedule):
+        self.schedule = schedule
+
         # Interpret the cronjob configuration as local time
         self._cron = croniter.croniter(schedule, datetime.datetime.now(utils.timeutil.tzlocal()))
 
@@ -87,6 +88,7 @@ class CronJob(object):
     def update(self, job):
         # Update the current job data (following a modified schedule or a plugin reinitialization)
         self.callback = job.callback
+        self.error = job.error
         if job.schedule != self.schedule:
             self.set_schedule(job.schedule)
 
@@ -129,23 +131,23 @@ class Crontab(object):
 
     def _reinit(self):
         self._plugin_callback = {}
+        self._formatters = {}
 
     def __init__(self):
         self._reinit()
         self._joblist = set()
         hookmanager.register("HOOK_PLUGINS_RELOAD", self._reinit)
-        hookmanager.register("HOOK_USER_DELETE", lambda user: self.delete(user=user))
 
     def _make_job(self, res):
-        err = func = None
         id, name, userid, schedule, ext_type, ext_id, base, runcnt, enabled, error_s = res
 
         func = self._plugin_callback.get(ext_type)
         if not func:
-            err = error.PrewikkaUserError(N_("Invalid job extension"), N_("Scheduled job with invalid extension type '%s'") % (ext_type))
-
-        if error_s and not(err):
+            err = error.PrewikkaUserError(N_("Invalid job extension"), N_("Scheduled job with invalid extension type '%s'", ext_type))
+        elif error_s:
             err = utils.json.loads(error_s)
+        else:
+            err = None
 
         if ext_id:
             ext_id = int(ext_id)
@@ -255,11 +257,8 @@ class Crontab(object):
     def add(self, name, schedule, user=None, ext_type=None, ext_id=None, enabled=True):
         return self.update(None, name=name, schedule=schedule, user=user, ext_type=ext_type, ext_id=ext_id, enabled=enabled)
 
-    def update_from_parameters(self, id, parameters=None, delete_disabled=False, **kwargs):
-        if parameters is None:
-            parameters = env.request.parameters
-
-        schedule = parameters.get("quick-schedule")
+    def update_from_parameters(self, id, delete_disabled=False, **kwargs):
+        schedule = env.request.parameters.get("quick-schedule")
         if schedule != "disabled":
             kwargs["schedule"] = schedule
             try:
@@ -270,7 +269,7 @@ class Crontab(object):
         elif delete_disabled:
             return crontab.delete(id=id, **kwargs)
 
-        crontab.update(id, name=parameters["name"], enabled=int(schedule != "disabled"), **kwargs)
+        crontab.update(id, enabled=int(schedule != "disabled"), **kwargs)
 
     def schedule(self, ext_type, name, schedule, _regfunc=None, enabled=True):
         if _regfunc:
@@ -278,12 +277,17 @@ class Crontab(object):
         else:
             return registrar.DelayedRegistrar.make_decorator("crontab", self.schedule, ext_type, name, schedule, enabled=enabled)
 
-    def setup(self, ext_type, _regfunc=None):
+    def setup(self, ext_type, _regfunc=None, formatter=None):
         if _regfunc:
             assert not(ext_type in self._plugin_callback)
             self._plugin_callback[ext_type] = _regfunc
+            self._formatters[ext_type] = formatter
         else:
-            return registrar.DelayedRegistrar.make_decorator("crontab", self.setup, ext_type)
+            return registrar.DelayedRegistrar.make_decorator("crontab", self.setup, ext_type, formatter=formatter)
+
+    def format(self, ext_type, name):
+        formatter = self._formatters.get(ext_type)
+        return N_(formatter, name) if formatter else name
 
 
 def format_schedule(x):
@@ -321,4 +325,5 @@ update = crontab.update
 delete = crontab.delete
 schedule = crontab.schedule
 setup = crontab.setup
+format = crontab.format
 update_from_parameters = crontab.update_from_parameters
