@@ -1,4 +1,4 @@
-# Copyright (C) 2016-2019 CS-SI. All Rights Reserved.
+# Copyright (C) 2016-2020 CS GROUP - France. All Rights Reserved.
 # Author: Abdel Elmili <abdel.elmili@c-s.fr>
 #
 # This file is part of the Prewikka program.
@@ -447,9 +447,6 @@ CriterionOperator = _CriterionEnum(
 
 
 class Criterion(json.JSONObject):
-    def _init(self, left, operator, right):
-        self.left, self.operator, self.right = left, operator, right
-
     def __init__(self, left=None, operator=None, right=None):
         json.JSONObject.__init__(self)
 
@@ -458,6 +455,9 @@ class Criterion(json.JSONObject):
 
         self._init(left, operator, right)
 
+    def _init(self, left, operator, right):
+        self.left, self.operator, self.right = left, operator, right
+
     def get_paths(self):
         res = set()
         if not self:
@@ -501,7 +501,7 @@ class Criterion(json.JSONObject):
         if not isinstance(value, compat.STRING_TYPES):
             value = text_type(value)
 
-        return "'%s'" % value.replace("\\", "\\\\").replace("'", "\\'")
+        return "'%s'" % value.replace("'", "\\'")
 
     def _criterion_to_string(self, path, operator, value):
         if value is None:
@@ -544,6 +544,13 @@ class Criterion(json.JSONObject):
         if not self or not self.operator.is_boolean:
             return self
 
+        if self.operator == CriterionOperator.NOT:
+            operand = self.right.flatten()
+            if operand.operator == self.operator:
+                return operand.operands[0]
+            else:
+                return AttrObj(operator=self.operator, operands=[operand])
+
         ret = AttrObj(operator=self.operator, operands=[])
 
         for operand in (self.left.flatten(), self.right.flatten()):
@@ -615,12 +622,18 @@ class Criterion(json.JSONObject):
 class DataProviderManager(pluginmanager.PluginManager):
     def __init__(self, autoupdate=False):
         pluginmanager.PluginManager.__init__(self, "prewikka.dataprovider.type", autoupdate=autoupdate)
+        pluginmanager.PluginManager("prewikka.dataprovider.backend", autoupdate=autoupdate)
 
         self._type_handlers = {}
         self._backends = {}
 
-    def load(self, autoupdate=False):
+    def load(self, reloading=False):
+        loaded = set()
+
         for k in self.keys():
+            if reloading and not self[k].error:
+                continue
+
             try:
                 p = self.initialize_plugin(self[k])
             except Exception:
@@ -628,8 +641,11 @@ class DataProviderManager(pluginmanager.PluginManager):
 
             p.dataprovider_type = k
             self._type_handlers[k] = p
+            loaded.add(p)
 
-        for plugin in pluginmanager.PluginManager("prewikka.dataprovider.backend", autoupdate=autoupdate):
+        for plugin in env.pluginmanager["prewikka.dataprovider.backend"]:
+            if reloading and not plugin.error:
+                continue
 
             if plugin.type not in self._type_handlers:
                 plugin.error = N_("No handler configured for '%s' datatype", plugin.type)
@@ -646,12 +662,15 @@ class DataProviderManager(pluginmanager.PluginManager):
                                               N_("Only one manager should be configured for '%s' backend", p.type))
 
             self._backends[p.type] = p
+            loaded.add(p)
 
         for p in self._type_handlers.values():
-            p.post_load()
+            if p in loaded:
+                p.post_load()
 
         for p in self._backends.values():
-            p.post_load()
+            if p in loaded:
+                p.post_load()
 
     @staticmethod
     def _parse_path(path):
@@ -713,7 +732,16 @@ class DataProviderManager(pluginmanager.PluginManager):
 
         return AttrObj(type=type, paths=paths, parsed_paths=parsed_paths, paths_types=paths_types, criteria=criteria.compile(type))
 
+    @staticmethod
+    def _check_limit_offset(limit, offset):
+        if limit < -1 or limit > 2**31 - 1:
+            raise DataProviderError("Limit parameter out of bounds")
+
+        if offset < 0 or offset > 2**31 - 1:
+            raise DataProviderError("Offset parameter out of bounds")
+
     def query(self, paths, criteria=None, distinct=False, limit=-1, offset=0, type=None, **kwargs):
+        self._check_limit_offset(limit, offset)
         o = self._normalize(type, paths, criteria)
 
         start = time.time()
@@ -729,6 +757,7 @@ class DataProviderManager(pluginmanager.PluginManager):
         return self._backends[type].get_by_id(id_)
 
     def get(self, criteria=None, order_by=["{backend}.{time_field}/order_desc"], limit=-1, offset=0, type=None):
+        self._check_limit_offset(limit, offset)
         o = self._normalize(type, order_by, criteria)
         return self._backends[o.type].get(o.criteria, o.paths, limit, offset)
 
