@@ -1,12 +1,28 @@
-# Copyright (C) 2016-2019 CS-SI. All Rights Reserved.
+# Copyright (C) 2016-2020 CS GROUP - France. All Rights Reserved.
 # Author: Abdel ELMILI <abdel.elmili@c-s.fr>
+#
+# This file is part of the Prewikka program.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2, or (at your option)
+# any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import datetime
 import prelude
 
-from prewikka import crontab, renderer, utils, version
+from prewikka import crontab, hookmanager, renderer, utils, version
 from prewikka.dataprovider import DataProviderBase, Criterion, CriterionOperator, ParserError, PathValue, InvalidPathError
 
 
@@ -163,7 +179,7 @@ class IDMEFAlertProvider(_IDMEFProvider):
         criteria = Criterion()
         age = int(config.get("age", 0))
         now = utils.timeutil.utcnow()
-        for severity in ("info", "low", "medium", "high"):
+        for severity in (None, "info", "low", "medium", "high"):
             days = int(config.get(severity, age))
             if days < 1:
                 continue
@@ -174,7 +190,8 @@ class IDMEFAlertProvider(_IDMEFProvider):
         if not criteria:
             return
 
-        env.dataprovider.delete(criteria, type="alert")
+        if not list(hookmanager.trigger("HOOK_CRON_DELETE", criteria, "alert")):
+            env.dataprovider.delete(criteria, type="alert")
 
 
 class IDMEFHeartbeatProvider(_IDMEFProvider):
@@ -196,4 +213,6 @@ class IDMEFHeartbeatProvider(_IDMEFProvider):
             return
 
         criteria = Criterion("heartbeat.create_time", "<", utils.timeutil.utcnow() - datetime.timedelta(days=days))
-        env.dataprovider.delete(criteria, type="heartbeat")
+
+        if not list(hookmanager.trigger("HOOK_CRON_DELETE", criteria, "heartbeat")):
+            env.dataprovider.delete(criteria, type="heartbeat")
