@@ -4,10 +4,28 @@ String.prototype.capitalize = function() {
     return this.charAt(0).toUpperCase() + this.slice(1);
 };
 
+/*
+ * Shamelessly stolen from stackOverflow.
+ */
+String.prototype.formatUnicorn = String.prototype.formatUnicorn || function () {
+    var str = this.toString();
+    if (arguments.length) {
+        var t = typeof arguments[0];
+        var key;
+        var args = ("string" === t || "number" === t) ?
+            Array.prototype.slice.call(arguments)
+            : arguments[0];
+
+        for (key in args) {
+            str = str.replace(new RegExp("\\{" + key + "\\}", "gi"), args[key]);
+        }
+    }
+
+    return str;
+};
+
 
 $(function() {
-  var $cache = null;
-  var $cachef = null;
 
   $.fn.serializeObject = function() {
       var data = {};
@@ -19,21 +37,6 @@ $(function() {
       return data;
   }
 
-  $.fn.popupUnique = function(animshow, animhide) {
-        if ( $cache && this.is(':visible') ) {
-                $cachef($cache);
-                $cache = null;
-        } else {
-                animshow(this);
-                if ( $cache )
-                        $cachef($cache);
-                $cachef = animhide;
-                $cache = this;
-        }
-
-        return false;
-  };
-
   $.fn.check = function(mode){
         return this.each(function() { this.checked = mode; } );
   };
@@ -98,10 +101,6 @@ $(function() {
       });
   });
 
-  $(document).on("click", ".popup_menu_toggle", function(){
-    $(this).next().popupUnique(function(data){data.show('fast'); data.css('display','block')}, function(data){data.hide('fast')});
-  });
-
   $(document).on("show.bs.popover", '[data-toggle="popover"]', function() {
     $('[data-toggle="popover"]').not(this).popover("hide");
   });
@@ -217,31 +216,6 @@ $(function() {
       _position_dropdown($(this), ".dropdown-menu");
   });
 
-  $(document).on('click', '.popup_menu_dynamic', function() {
-      $(this).removeClass("popup_menu_dynamic");
-
-      if ( ! $(this).data("popup-url") )
-          return;
-
-      var popup_menu = $(this).next(".popup_menu");
-      popup_menu.append($("<span>", {
-        "class": "popup_menu_loading",
-        "text": "Loading..."
-      }));
-
-      prewikka_ajax({
-          prewikka: { spinner: false },
-          type: "GET",
-          url: $(this).data("popup-url"),
-          success: function(data) {
-              popup_menu.find(".popup_menu_loading").remove();
-              $.each(data, function(i, node) {
-                  popup_menu.append(node.toHTML());
-              });
-          }
-      });
-  });
-
   $(document).on('click', '.prewikka-notification .close', function() {
     $(this).parent().parent().remove();
   });
@@ -467,7 +441,7 @@ function prewikka_getRenderSize(container, options)
         else {
             size[1] = options.height;
             if ( ! size[1] )
-                size[1] = $("#_main_viewport").height();
+                size[1] = $("#_main_viewport").height() - $(parent).offset().top + $("#_main_viewport").offset().top;
         }
 
         document.body.style.overflow = overflow_backup;
@@ -500,6 +474,7 @@ function prewikka_autocomplete(selector, options) {
     options = options || {};
     $(selector).select2_container({
         allowClear: !!options.allow_empty,
+        dropdownAutoWidth: !!options.dropdownAutoWidth,
         placeholder: options.placeholder || "",
         ajax: {
             url: options.url,
