@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2019 CS-SI. All Rights Reserved.
+# Copyright (C) 2004-2020 CS GROUP - France. All Rights Reserved.
 # Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
@@ -293,15 +293,15 @@ class Parameters(dict):
 
 
 class GeneralParameters(Parameters):
-    def register(self):
-        Parameters.register(self)
-        mainmenu._register_parameters(self)
-
     def __init__(self, vobj, kw):
         # Only allow parameters saving if the view is a primary route (has a view_menu entry)
         Parameters.__init__(self, vobj, _save=bool(vobj.view_menu), **kw)
         self.normalize()
 
+    def register(self):
+        Parameters.register(self)
+        mainmenu._register_parameters(self)
+
 
 class ViewResponse(response.PrewikkaResponse):
     def __init__(self, data, **kwargs):
@@ -343,21 +343,6 @@ class _ViewDescriptor(object):
 
     view_csrf_exempt = False
 
-    @property
-    def view_others_permissions(self):
-        return self.view_permissions
-
-    @view_others_permissions.setter
-    def view_others_permissions(self, permissions):
-        self.view_permissions = permissions
-
-    @classmethod
-    def _handle_attributes(cls, *args):
-        for permission in cls.view_permissions:
-            usergroup.ALL_PERMISSIONS.declare(permission)
-
-        pluginmanager.PluginBase._handle_attributes.__func__(cls, *args)
-
     def __init__(self, baseview=None):
         if baseview:
             self.view_base = baseview
@@ -376,6 +361,21 @@ class _ViewDescriptor(object):
         self.view_groups = set(self.view_groups)
         self.view_permissions = set(self.view_permissions)
 
+    @property
+    def view_others_permissions(self):
+        return self.view_permissions
+
+    @view_others_permissions.setter
+    def view_others_permissions(self, permissions):
+        self.view_permissions = permissions
+
+    @classmethod
+    def _handle_attributes(cls, *args):
+        for permission in cls.view_permissions:
+            usergroup.ALL_PERMISSIONS.declare(permission)
+
+        pluginmanager.PluginBase._handle_attributes.__func__(cls, *args)
+
     def _setup_dataset_default(self, dataset):
         dataset["document"] = utils.AttrObj()
         dataset["document"].base_url = utils.iri2uri(env.request.web.get_baseurl())
@@ -446,9 +446,6 @@ class _ViewDescriptor(object):
 class _View(_ViewDescriptor, registrar.DelayedRegistrar):
     view_id = None
 
-    def render(self):
-        pass
-
     def __init__(self):
         if not self.view_id:
             self.view_id = self.__class__.__name__.lower()
@@ -459,6 +456,9 @@ class _View(_ViewDescriptor, registrar.DelayedRegistrar):
 
         _ViewDescriptor.__init__(self)
 
+    def render(self):
+        pass
+
     def make_parameters(self, _view_descriptor=None, criteria=None, **kwargs):
         values = {}
         view = _view_descriptor or self
@@ -492,22 +492,41 @@ class View(_View, pluginmanager.PluginBase):
         pluginmanager.PluginBase.__init__(self)
 
 
-def route(path, method=_SENTINEL, methods=["GET"], permissions=[], menu=None, label=None, defaults={}, endpoint=None, datatype=None, priority=0, keywords=set(), help=None, parameters=_SENTINEL):
+def route(path, method=_SENTINEL, methods=["GET"], permissions=[], menu=None, label=None, defaults={},
+          endpoint=None, datatype=None, priority=0, keywords=set(), help=None, parameters=_SENTINEL, csrf_exempt=False):
     usergroup.ALL_PERMISSIONS.declare(permissions)
 
     if method is not _SENTINEL:
         ret = env.viewmanager._add_route(path, method, methods=methods, permissions=permissions, menu=menu, label=label,
                                          defaults=defaults, endpoint=endpoint, datatype=datatype, priority=priority,
-                                         keywords=keywords, help=help, parameters=parameters)
+                                         keywords=keywords, help=help, parameters=parameters, csrf_exempt=csrf_exempt)
     else:
         ret = registrar.DelayedRegistrar.make_decorator("route", env.viewmanager._add_route, path, methods=methods, permissions=permissions,
                                                         menu=menu, label=label, defaults=defaults, endpoint=endpoint, datatype=datatype, keywords=keywords,
-                                                        priority=priority, help=help, parameters=parameters)
+                                                        priority=priority, help=help, parameters=parameters, csrf_exempt=csrf_exempt)
 
     return ret
 
 
 class ViewManager(registrar.DelayedRegistrar):
+    def __init__(self):
+        registrar.DelayedRegistrar.__init__(self)
+        self._init()
+
+        builtins.url_for = self.url_for
+
+    def _init(self):
+        _URL_ADAPTER_CACHE.clear()
+
+        self._references = {}
+        self._views_endpoints = {}
+
+        self._route_override = {}
+        for i in _ROUTE_OVERRIDE_TYPE:
+            self._route_override[i] = {}
+
+        self._rule_map = Map(converters={'list': ListConverter})
+
     def get(self, datatype=None, keywords=None):
         views = self._references.get(datatype, []) if datatype else self._views_endpoints.values()
         return sorted(filter(lambda x: set(keywords or []).issubset(x.view_keywords), views), key=lambda x: x.view_priority)
@@ -576,7 +595,8 @@ class ViewManager(registrar.DelayedRegistrar):
         self._views_endpoints[view.view_endpoint] = view
         self._rule_map.add(rule)
 
-    def _add_route(self, path, method=None, methods=["GET"], permissions=[], menu=None, label=None, defaults={}, endpoint=None, datatype=None, priority=0, keywords=set(), help=None, parameters=None):
+    def _add_route(self, path, method=None, methods=["GET"], permissions=[], menu=None, label=None, defaults={},
+                   endpoint=None, datatype=None, priority=0, keywords=set(), help=None, parameters=None, csrf_exempt=False):
         v = _ViewDescriptor(baseview=method.__self__)
 
         v.render = method
@@ -587,6 +607,7 @@ class ViewManager(registrar.DelayedRegistrar):
         v.view_datatype = datatype
         v.view_priority = priority
         v.view_keywords = set(keywords)
+        v.view_csrf_exempt = csrf_exempt
 
         if help:
             v.view_help = help
@@ -606,11 +627,7 @@ class ViewManager(registrar.DelayedRegistrar):
         self._init()
         self.get_baseview()
 
-        for view_class in pluginmanager.PluginManager("prewikka.views", autoupdate):
-            try:
-                pluginmanager.PluginManager.initialize_plugin(view_class)
-            except Exception:
-                continue
+        pluginmanager.PluginManager("prewikka.views", autoupdate).load()
 
     @property
     def url_adapter(self):
@@ -624,24 +641,6 @@ class ViewManager(registrar.DelayedRegistrar):
 
         return ad
 
-    def _init(self):
-        _URL_ADAPTER_CACHE.clear()
-
-        self._references = {}
-        self._views_endpoints = {}
-
-        self._route_override = {}
-        for i in _ROUTE_OVERRIDE_TYPE:
-            self._route_override[i] = {}
-
-        self._rule_map = Map(converters={'list': ListConverter})
-
-    def __init__(self):
-        registrar.DelayedRegistrar.__init__(self)
-        self._init()
-
-        builtins.url_for = self.url_for
-
     def url_for(self, endpoint, _default=_SENTINEL, **kwargs):
         view = self.get_view(endpoint=endpoint)
         if not view:
