@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2015-2019 CS-SI. All Rights Reserved.
+# Copyright (C) 2015-2020 CS GROUP - France. All Rights Reserved.
 # Author: Sélim Menouar <selim.menouar@c-s.fr>
 #
 # This file is part of the Prewikka program.
@@ -23,7 +23,6 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import collections
-import copy
 import csv
 import datetime
 import functools
@@ -35,8 +34,9 @@ import re
 from prewikka.utils import json
 from prewikka import error, history, hookmanager, mainmenu, resource, response, template, utils, view
 from prewikka.dataprovider import Criterion, ResultObject
+from prewikka.dataprovider.pathparser import PathParser
 from prewikka.dataprovider.parsers import criteria, lucene
-from prewikka.localization import format_datetime
+from prewikka.localization import format_datetime, format_timedelta
 from prewikka.renderer import RendererItem
 from prewikka.statistics import ChronologyChart, DiagramChart, Query
 
@@ -61,6 +61,7 @@ class DataSearchParameters(view.Parameters):
         self.optional("query_mode", text_type, save=True)
         self.optional("editable", int, save=True)
         self.optional("condensed", int, save=True)
+        self.optional("expert", int, save=True)
         self.optional("jqgrid_params_datasearch_table", json.loads, default={}, persist=True)
 
 
@@ -99,7 +100,7 @@ class Formatter(object):
     ignore_fields = frozenset()
 
     _converters = {
-        datetime.datetime: lambda f, r, o: resource.HTMLNode("span", format_datetime(o), **{"data-field": f.field})
+        datetime.datetime: lambda f, r, o: resource.HTMLNode("span", format_datetime(o), **{"data-field": f.field, "data-value": o})
     }
 
     def __init__(self, data_type):
@@ -138,40 +139,15 @@ class Formatter(object):
 class QueryParser(object):
     path_prefix = "{backend}."
 
-    def _prepare_groupby_query(self, groupby):
-        self._path = ["count(1)"]
-
-        groupby = set(groupby)
-        ogroup = list(groupby - set(_TEMPORAL_VALUES))
-        tgroup = list(groupby & set(_TEMPORAL_VALUES))
-        self.groupby = ogroup + tgroup
-
-        for field in ogroup:
-            if field not in self._parent.path_translate:
-                self._path.append('%s%s/group_by' % (self.path_prefix, field))
-            else:
-                for i in self._parent.path_translate[field][0]:
-                    self._path.append('%s/group_by' % (i))
-
-        if not tgroup:
-            self._path[0] = "count(1)/order_desc"
-            return
-
-        if len(tgroup) > 1:
-            raise error.PrewikkaUserError(N_("Time group error"), N_("Only one time unit can be specified in a groupby query"))
-
-        self._time_group = tgroup[0]
-        self._date_selection_index = len(self._path)
-        self._path += self._time_selection(self._time_group)
-
-    def __init__(self, query, parent, groupby=[], offset=0, limit=50):
+    def __init__(self, query, parent, groupby=[], orderby=[], offset=0, limit=50):
         self.type = parent.type
         self.query = query
         self.offset = offset
         self.limit = limit
         self.groupby = []
+        self._sort_order = ["%s.%s/order_%s" % (self.type, field, order) for field, order in orderby]
         self._time_group = None
-        self._path = []
+        self._paths = collections.OrderedDict()
         self._result = None
         self._parent = parent
         self._date_selection_index = None
@@ -180,9 +156,47 @@ class QueryParser(object):
         self.all_criteria = self.criteria + env.request.menu.get_criteria()
 
         if groupby:
-            self._prepare_groupby_query(groupby)
+            self._prepare_groupby_query(groupby, orderby)
         else:
-            self._path = ['%s%s' % (self.path_prefix, field) for field in self._parent.all_fields]
+            self._paths.update((field, '%s%s' % (self.path_prefix, field)) for field in self._parent.all_fields)
+            self._handle_order(orderby)
+
+        if self.query:
+            # Make sure to use printable characters in the search bar
+            self.query = "".join(chr(x) for x in bytearray(self.query.encode("unicode-escape")))
+
+    def _prepare_groupby_query(self, groupby, orderby):
+        self._paths["_aggregation"] = "count(1)"
+
+        groupby = set(groupby)
+        ogroup = list(groupby - set(_TEMPORAL_VALUES))
+        tgroup = list(groupby & set(_TEMPORAL_VALUES))
+        self.groupby = ogroup + tgroup
+
+        for field in ogroup:
+            if field not in self._parent.path_translate:
+                self._paths[field] = '%s%s/group_by' % (self.path_prefix, field)
+            else:
+                for i in self._parent.path_translate[field][0]:
+                    self._paths[field] = '%s/group_by' % i
+
+        self._handle_order(orderby)
+
+        if not tgroup:
+            return
+
+        if len(tgroup) > 1:
+            raise error.PrewikkaUserError(N_("Time group error"), N_("Only one time unit can be specified in a groupby query"))
+
+        self._time_group = tgroup[0]
+        self._date_selection_index = len(self._paths)
+        self._paths.update(("_time_unit_%d" % i, path) for i, path in enumerate(self._time_selection(self._time_group)))
+
+    def _handle_order(self, orderby):
+        for field, order in orderby:
+            path = self._paths.pop(field, "%s%s" % (self.path_prefix, field))
+            separator = "," if "/" in path else "/"
+            self._paths[field] = "%s%sorder_%s" % (path, separator, order)
 
     def get_result(self):
         if self._result:
@@ -215,7 +229,7 @@ class QueryParser(object):
             if not value:
                 return "-%s:[* TO *]" % path
 
-            return "%s:%s" % (path, cls._lucene_escape(value))
+            return "%s.exact:%s" % (path, cls._lucene_escape(value))
 
         return text_type(Criterion(path, "==", value))
 
@@ -304,6 +318,15 @@ class QueryParser(object):
 
         return functools.reduce(lambda x, y: f(x, y), (Criterion(i, self._fix_operator(i, op), right) for i in paths))
 
+    def get_index(self, field):
+        if field in _TEMPORAL_VALUES:
+            return self._date_selection_index
+
+        return list(self._paths).index(field)
+
+    def get_paths(self):
+        return list(self._paths.values())
+
     def get_criteria(self, query):
         if not query:
             return Criterion()
@@ -326,21 +349,10 @@ class QueryParser(object):
 
         return selection
 
-    def add_order(self, field, order="asc"):
-        if order not in ("asc, desc"):
-            return
-
-        try:
-            idx = self._path.index("{backend}.%s" % field)
-        except ValueError:
-            self._path.append("{backend}.%s/order_%s" % (field, order))
-        else:
-            self._path[idx] += "/order_%s" % order
-
     def _diagram_data(self, cview, step):
         """Generator for the diagram chart"""
         for result in self.get_result():
-            value = result[0]
+            count = result[0]
             labels = result[1:]
 
             link = self.get_groupby_link(self.groupby, labels, step, cview=cview)
@@ -348,10 +360,10 @@ class QueryParser(object):
             if self._date_selection_index:
                 labels[-1] = labels[-1].strftime(step.unit_format)
 
-            yield RendererItem(value or "", ", ".join((text_type(x) for x in labels)), link)
+            yield RendererItem(count, tuple(labels), link)
 
     def _query(self):
-        return env.dataprovider.query(self._path, self.all_criteria, limit=self.limit, offset=self.offset, type=self.type)
+        return env.dataprovider.query(self.get_paths(), self.all_criteria, limit=self.limit, offset=self.offset, type=self.type)
 
     def _groupby_query(self):
         return self._query()
@@ -372,10 +384,11 @@ class DataSearch(view.View):
     path_translate = {}
     default_columns = {}
     lucene_search_fields = []
+    expert_enabled = False
     _extra_resources = []
 
     criterion_config["lucene"] = {
-        "format": 'operator + path + ":" + value',
+        "format": '{operator}{path}:{value}',
         "operators": {
             "equal": "",
             "notequal": "-",
@@ -386,7 +399,7 @@ class DataSearch(view.View):
         }
     }
     criterion_config["criterion"] = {
-        "format": 'path + " " + operator + " " + value',
+        "format": '{path} {operator} {value}',
         "operators": {
             "equal": "=",
             "notequal": "!=",
@@ -397,6 +410,35 @@ class DataSearch(view.View):
         }
     }
 
+    def __init__(self):
+        env.dataprovider.check_datatype(self.type)
+
+        self._formatter = self.formatter(self.type)
+
+        self.all_fields = []
+        self._main_fields = list(self.default_columns.keys())
+        self.fields_info = collections.OrderedDict()
+        self.columns_properties = collections.OrderedDict()
+
+        self._prepare_fields()
+        view.View.__init__(self)
+
+        hookmanager.register("HOOK_LOAD_HEAD_CONTENT", [resource.CSSLink("datasearch/css/datasearch.css")])
+
+        section = self.section or env.dataprovider.get_label(self.type)
+        tabs = self.tabs or (N_("Forensic"), N_("Dashboard"))
+
+        view.route("/%s/forensic/ajax_timeline" % self.name, self.ajax_timeline)
+        view.route("/%s/forensic/ajax_table" % self.name, self.ajax_table)
+        view.route("/%s/forensic/ajax_details" % self.name, self.ajax_details)
+        view.route("/%s/forensic/ajax_infos" % self.name, self.ajax_infos)
+        view.route("/%s/forensic/ajax_groupby" % self.name, self.ajax_groupby)
+        view.route("/%s/forensic/csv_download" % self.name, self.csv_download, methods=["POST"])
+        view.route("/%s/forensic" % self.name, self.forensic, menu=(section, tabs[0]), keywords=["listing", "inheritable"],
+                   datatype=self.type, priority=1, help="#%sforensic" % self.type, methods=["POST", "GET"])
+        view.route("/%s/dashboard" % self.name, self.dashboard, menu=(section, tabs[1]),
+                   datatype=self.type, help="#%sdashboard" % self.type, methods=["POST", "GET"])
+
     def _get_fields(self):
         return env.dataprovider.get_paths(self.type)
 
@@ -419,52 +461,27 @@ class DataSearch(view.View):
             pi.filterable = pi.type is not datetime.datetime
             pi.groupable = pi.type is not object
 
-            pi.column_index = self._column_index
-            self._column_index += 1
-
             cprop = self._get_column_property(field, pi)
             if cprop:
                 self.columns_properties[field] = cprop
 
-    def __init__(self):
-        env.dataprovider.check_datatype(self.type)
-
-        self._formatter = self.formatter(self.type)
-        self._column_index = 0
-
-        self.all_fields = []
-        self._main_fields = list(self.default_columns.keys())
-        self.fields_info = collections.OrderedDict()
-        self.columns_properties = collections.OrderedDict()
-
-        self._prepare_fields()
-        view.View.__init__(self)
-
-        hookmanager.register("HOOK_LOAD_HEAD_CONTENT", [resource.CSSLink("datasearch/css/datasearch.css")])
-
-        section = self.section or env.dataprovider.get_label(self.type)
-        tabs = self.tabs or (N_("Forensic"), N_("Dashboard"))
-
-        view.route("/%s/forensic/ajax_timeline" % self.name, self.ajax_timeline)
-        view.route("/%s/forensic/ajax_table" % self.name, self.ajax_table)
-        view.route("/%s/forensic/ajax_details" % self.name, self.ajax_details)
-        view.route("/%s/forensic/ajax_infos" % self.name, self.ajax_infos)
-        view.route("/%s/forensic/csv_download" % self.name, self.csv_download, methods=["POST"])
-        view.route("/%s/forensic" % self.name, self.forensic, menu=(section, tabs[0]), keywords=["listing", "inheritable"],
-                   datatype=self.type, priority=1, help="#%sforensic" % self.type, methods=["POST", "GET"])
-        view.route("/%s/dashboard" % self.name, self.dashboard, menu=(section, tabs[1]),
-                   datatype=self.type, help="#%sdashboard" % self.type, methods=["POST", "GET"])
-
     def _set_common(self, dataset):
         view.View.render(self)
 
         dataset["backend"] = self.type
         dataset["limit"] = env.request.parameters["limit"]
 
-        # Deepcopy is necessary because the forensic template updates the object values
-        dataset["columns_properties"] = colsprop = copy.deepcopy(self.columns_properties)
+        dataset["columns_properties"] = columns = collections.OrderedDict()
+        right_columns = []
         for prop, finfo, func in filter(None, self._trigger_datasearch_hook("EXTRA_COLUMN")):
-            colsprop[prop.label] = COLUMN_PROPERTIES(**dict(prop))
+            if prop.position == "left":
+                columns[prop.label] = COLUMN_PROPERTIES(**prop)
+            else:
+                right_columns.append(prop)
+
+        columns.update(self.columns_properties)
+        for prop in right_columns:
+            columns[prop.label] = COLUMN_PROPERTIES(**prop)
 
         dataset["separators"] = self._formatter.highlighter.get_separators()
         dataset["criterion_config"] = self.criterion_config
@@ -520,7 +537,8 @@ class DataSearch(view.View):
                                               limit=env.request.parameters["limit"],
                                               parent=self)
         dataset["extra_resources"] = self._extra_resources
-        dataset["common_paths"] = {path.split(".", 1)[-1]: _(label).lower() for label, path in env.dataprovider.get_common_paths(self.type, index=True)}
+        dataset["common_paths"] = {path.split(".", 1)[-1]: _(label) for label, path in env.dataprovider.get_common_paths(self.type, index=True)}
+        dataset["expert_enabled"] = self.expert_enabled
 
         return view.ViewResponse(dataset)
 
@@ -529,19 +547,29 @@ class DataSearch(view.View):
         if query:
             history.save(env.request.user, "%s_form_search" % self.type, query)
 
-        search = self.query_parser(query, groupby=env.request.parameters.get("groupby"),
-                                   offset=(page - 1) * limit, limit=limit, parent=self)
+        groupby = env.request.parameters.get("groupby")
+        orderby = []
 
-        field = env.request.parameters.get("sort_index")
-        order = env.request.parameters.get("sort_order")
-        if field in self.all_fields:
-            search.add_order(field, order)
-        elif field in self.path_translate:
-            search.add_order(self.path_translate[field][0][0].split(".", 1)[-1], order)
-        else:
-            search.add_order(self.sort_path_default, "desc")
+        # @HACK free-jqGrid messes the parameters badly when sorting on multiple columns simultaneously
+        sort_settings = "%s %s" % (env.request.parameters.get("sort_index", ""), env.request.parameters.get("sort_order", ""))
+        for sort_setting in sort_settings.split(','):
+            try:
+                field, order = sort_setting.split()
+            except ValueError:
+                pass
+            else:
+                if order not in ("asc", "desc"):
+                    pass
+                elif field in self.path_translate:
+                    orderby.append((self.path_translate[field][0][0].split(".", 1)[-1], order))
+                elif field == "_aggregation" or PathParser.unindex_path(field) in self.all_fields:
+                    orderby.append((field, order))
 
-        return search
+        if not orderby:
+            orderby.append(("_aggregation" if groupby else self.sort_path_default, "desc"))
+
+        return self.query_parser(query, groupby=groupby, orderby=orderby,
+                                 offset=(page - 1) * limit, limit=limit, parent=self)
 
     def csv_download(self):
         grid = utils.json.loads(env.request.parameters["datasearch_grid"], object_pairs_hook=collections.OrderedDict)
@@ -565,12 +593,13 @@ class DataSearch(view.View):
             <script type="text/javascript">%s</script>
             """ % (data["html"], data["script"] or "")))
 
-    def _get_default_cells(self, obj):
+    def _get_default_cells(self, obj, search):
         r = {}
 
         for fname, cprop in self.columns_properties.items():
             finfo = self.fields_info[fname]
-            r[fname] = self._formatter.format(finfo, obj, obj[finfo.column_index])
+            index = getattr(finfo, "column_index", search.get_index(fname))
+            r[fname] = self._formatter.format(finfo, obj, obj[index])
 
         return r
 
@@ -583,12 +612,12 @@ class DataSearch(view.View):
         extracol = list(filter(None, self._trigger_datasearch_hook("EXTRA_COLUMN")))
 
         for i, obj in enumerate(results):
-            cells = self._get_default_cells(obj)
+            cells = self._get_default_cells(obj, search)
             for prop, finfo, func in extracol:
                 if isinstance(obj, ResultObject):
                     ret = func(obj, extradata)
                 else:
-                    ret = func({"%s.%s" % (self.type, f): v for f, v in zip(self.all_fields, obj)}, extradata)
+                    ret = func({env.dataprovider.format_path(f.split("/")[0], type=self.type): v for f, v in zip(search.get_paths(), obj)}, extradata)
 
                 if finfo:
                     cells[prop.name] = self._formatter.format(finfo, obj, ret)
@@ -627,6 +656,39 @@ class DataSearch(view.View):
 
         return response.PrewikkaResponse({"infos": infos})
 
+    def ajax_groupby(self):
+        limit = int(env.request.parameters["limit"])
+        page = int(env.request.parameters.get("page", 1))
+
+        search = self._prepare(page, limit)
+        step = search.get_step()
+        results = search.get_result()
+        resrows = []
+
+        # We need to reorder results according to what is expected
+        permutation = [search.get_index(f) for f in ["_aggregation"] + search.groupby]
+
+        for i, result in enumerate(results):
+            values = [result[permutation[idx]] for idx in range(len(result))]
+
+            cells = {}
+            for idx, group in enumerate(search.groupby):
+                label = values[idx + 1]
+                if isinstance(label, datetime.datetime):
+                    label = label.strftime(step.unit_format)
+                elif isinstance(label, datetime.timedelta):
+                    label = format_timedelta(label)
+
+                link = search.get_groupby_link([group], [values[idx + 1]], step, cview='.forensic')
+                cells[group] = resource.HTMLNode("a", label, href=link)
+
+            link = search.get_groupby_link(search.groupby, values[1:], step, cview='.forensic')
+            cells["_aggregation"] = resource.HTMLNode("a", values[0], href=link)
+            resrows.append({"id": text_type(i), "cell": cells})
+
+        total = (page if len(resrows) < limit else page + 1) * limit
+        return utils.viewhelpers.GridAjaxResponse(resrows, total).add_html_content(mainmenu.HTMLMainMenu(update=True))
+
 
 class ResultDatetimeIterator(object):
     def __init__(self, results, date_selection_index):
