@@ -9,7 +9,7 @@ function DataSearchPage(backend, criterion_config, criterion_config_default, sep
     function lucene_need_quotes(value)
     {
         /*
-         * We add "/"" to the lucene escape characters as ELK interpret them as a regexp
+         * We add "/"" to the lucene escape characters as Elasticsearch interprets them as a regexp
          */
         return /[/\s+\-!(){}[\]^"~*?\:\\]|&&|\|\|/g.test(value);
     }
@@ -34,71 +34,55 @@ function DataSearchPage(backend, criterion_config, criterion_config_default, sep
     function quote(value)
     {
         if ( need_quotes(value) )
-            return '"' + value.replace(/(["\\])/g, '\\$1') + '"';
+            return '"' + value.replace(/(["])/g, '\\$1') + '"';
 
         return value;
     }
 
-    function _criterion(path, operator, value)
+    function criterion(path, opdef, value)
     {
-        if ( value == undefined )
-            value = "";
-
-        operator = criterion_config[criterion_config_default].operators[operator];
-
-        return eval(criterion_config[criterion_config_default].format);
-    }
-
-    function lucene_criterion(path, operator, value)
-    {
-        // We can't mix wildcards and quotes in Lucene
-        return _criterion(path, operator, (operator == "substr" && ! need_quotes(value)) ? value + "*" : quote(value));
-    }
-
-    function idmef_criterion(path, operator, value)
-    {
-        return _criterion(path, operator, quote(value));
-    }
-
-    function criterion(path, operator, value)
-    {
-        var ret;
-
-        if ( criterion_config_default == "criterion" )
-            ret = idmef_criterion(path, operator, value);
-        else
-            ret = lucene_criterion(path, operator, value);
-
-        return ret;
-    }
-
-    function lucene_criterion_regex(path, operator, value)
-    {
-        var ret;
-        var opstr = criterion_config[criterion_config_default].operators[operator];
+        var operator = criterion_config[criterion_config_default].operators[opdef];
 
         if ( value == undefined )
             value = "";
 
-        ret = escapeRegex(opstr + path + ":" + value);
+        value = _escape(opdef, value);
+        if ( need_quotes(value) )
+            value = quote(value)
 
-        if ( operator == "equal" || operator == "substr" )
+        else if ( (operator == "substr" || operator == "notsubstr") && criterion_config_default == "lucene" )
+                value += "*";
+
+        return criterion_config[criterion_config_default].format.formatUnicorn({path: path, operator: operator, value: value});
+    }
+
+    function lucene_criterion_regex(path, opdef, value)
+    {
+        var ret;
+        var operator = criterion_config[criterion_config_default].operators[opdef];
+
+        if ( value == undefined )
+            value = "";
+
+        ret = operator + path + ":" + _escape(opdef, escapeRegex(value.toString()));
+        if ( opdef == "equal" || opdef == "substr" )
             ret = "[^-]" + ret;
 
         return ret;
     }
 
-    function idmef_criterion_regex(path, operator, value)
+    function idmef_criterion_regex(path, opdef, value)
     {
+        var operator;
         if ( value == undefined )
             value = "";
 
         if ( operator )
-            operator = criterion_config[criterion_config_default].operators[operator];
+            operator = criterion_config[criterion_config_default].operators[opdef];
         else
             operator = "\\s*[=<>]+\\s*";
 
-        return escapeRegex(path) + operator + escapeRegex(value.toString());
+        return escapeRegex(path) + operator + _escape(opdef, escapeRegex(value.toString()));
     }
 
     function criterion_regex(path, operator, value)
@@ -137,12 +121,40 @@ function DataSearchPage(backend, criterion_config, criterion_config_default, sep
         return $.trim(search);
     }
 
+    function _escape(operator, value)
+    {
+        var out = "";
+        var escaped = {'\0': '\\0', 0x07: '\\a', '\b': '\\b', '\f': '\\f', '\n': '\\n', '\r': '\\r', '\t': '\\t', '\v': '\\v'};
+
+        for ( var i = 0; i < value.length; ) {
+                var c = value.charAt(i);
+                var d = value.charCodeAt(i++);
+
+                if ( d in escaped )
+                        out += escaped[d];
+
+
+                else if ( (operator == "substr" || operator == "notsubstr") && (c == '*' || c == '?') )
+                        out += "\\" + c;
+
+                else if ( d > 31 && d < 127 )
+                        out += c;
+
+                else  {
+                        var x = d.toString(16)
+                        out += "\\x" + ((x.length == 1) ? "0" + x : x);
+                }
+        }
+
+        return out;
+    }
+
     function _add_to_input(field, operator, value, positive)
     {
-        value = String(value);
-
         var search;
 
+        value = String(value).replace('\\', '\\\\');
+
         if ( positive ) {
             search = sub_from_search(field, operator, null, false);
             search = sub_from_search(field, "notequal", value, positive, search);
@@ -184,7 +196,7 @@ function DataSearchPage(backend, criterion_config, criterion_config_default, sep
 
     function update_datasearch()
     {
-        set_postdata();
+        set_postdata("#datasearch_table", false);
 
         $("#datasearch_table").trigger("reloadGrid");
         render_timeline(true);
@@ -262,6 +274,7 @@ function DataSearchPage(backend, criterion_config, criterion_config_default, sep
         var endNode;
         var endPos;
         var textLen;
+        var textContent;
 
         // Do not change the selection when the popover is shown
         var visible = $("#PopoverOption").is(':visible');
@@ -289,14 +302,14 @@ function DataSearchPage(backend, criterion_config, criterion_config_default, sep
         // Find the word's starting position
         find_start:
         while ( true ) {
-            for ( ; startPos > 0; startPos-- ) {
-                if ( separators.word.indexOf(startNode.textContent[startPos - 1]) > -1 ) {
+            for ( textContent = startNode.textContent; startPos > 0; startPos-- ) {
+                if ( separators.word.indexOf(textContent[startPos - 1]) > -1 ) {
                     break find_start;
                 }
             }
             if ( startNode.parentNode.previousSibling !== null ) {
                 startNode = startNode.parentNode.previousSibling.firstChild;
-                startPos = startNode.textContent.length;
+                startPos = startNode.length;
             } else {
                 break find_start;
             }
@@ -305,8 +318,8 @@ function DataSearchPage(backend, criterion_config, criterion_config_default, sep
         // Find the word's ending position
         find_end:
         while ( true ) {
-            for ( textLen = endNode.length; endPos < textLen; endPos++ ) {
-                if ( separators.word.indexOf(endNode.textContent[endPos]) > -1 || separators.term.indexOf(endNode.textContent[endPos]) > -1 ) {
+            for ( textLen = endNode.length, textContent = endNode.textContent; endPos < textLen; endPos++ ) {
+                if ( separators.word.indexOf(textContent[endPos]) > -1 || separators.term.indexOf(textContent[endPos]) > -1 ) {
                     break find_end;
                 }
             }
@@ -388,8 +401,7 @@ function DataSearchPage(backend, criterion_config, criterion_config_default, sep
         var td = node.closest("td").first();
         var selected_field = node.closest("[data-field]");
         var selected_value = node.closest("[data-value]");
-        var next_char = node[0].nextSibling.nodeValue[0];
-        var selected_operator = separators.term.indexOf(next_char) === -1 ? "equal" : "substr";
+        var selected_operator = (node.text() == selected_field.text()) ? "equal" : "substr";
 
         selected_value = selected_value.length > 0 ? selected_value.data("value") : selvalue;
         selected_field = selected_field.data("field");
@@ -422,31 +434,42 @@ function DataSearchPage(backend, criterion_config, criterion_config_default, sep
         });
 
         $("#PopoverOption a.groupby_search").attr("href", prewikka_location().href + "?groupby[]=" + selected_field);
-        $("#PopoverOption .groupby_search span").text(common_paths[selected_field] || selected_field);
+        $("#PopoverOption .groupby_search span").text((common_paths[selected_field] || selected_field).toLowerCase());
         $("#PopoverOption").show();
 
-        var oca_position = "bottom";
         var popover = $("#PopoverOption .popover");
-        var top = offset.top + node.height();
-        var left = offset.left - popover.width() / 2 + node.width() / 2;
+        var top, left = offset.left - popover.width() / 2 + node.width() / 2;
 
         popover.find(".dropdown-submenu").removeClass("pull-left");
+        popover.removeClass("bottom top left right menu-left");
 
-        if ( offset.left + node.width() / 2 + popover.width() > window.innerWidth ) {
-            top -= popover.height() / 2 + node.height() / 2;
-            left = offset.left - popover.width();
-            oca_position = "left";
-            popover.find(".dropdown-submenu").addClass("pull-left");
-        } else if ( offset.left - node.width() / 2 - popover.width() / 2 < 0 ) {
-            top -= popover.height() / 2 + node.height() / 2;
+        if ( left < 0 ) {
+            /* Handle the case of a narrow column near the left side of the grid */
+            popover.addClass("right");
+            top = offset.top - popover.height() / 2 + node.height() / 2;
             left = offset.left + node.width();
-            oca_position = "right";
-        } else if ( offset.top + node.height() + popover.height() > window.innerHeight ) {
+        }
+        else if ( left + popover.width() > window.innerWidth ) {
+            /* Handle the case of a narrow column near the right side of the grid */
+            popover.addClass("left");
+            top = offset.top - popover.height() / 2 + node.height() / 2;
+            left = offset.left - popover.width();
+        }
+        /* Otherwise, expand the menu upwards or downwards, and the submenu
+         * leftwards or rightwards, according to where the most space is available */
+        else if ( window.innerHeight - (offset.top + node.height()) > offset.top ) {
+            popover.addClass("bottom");
+            top = offset.top + node.height();
+        }
+        else {
+            popover.addClass("top");
             top = offset.top - (node.height() / 2 + popover.height());
-            oca_position = "top";
+        }
+        if ( window.innerWidth - (offset.left + node.width()) < offset.left ) {
+            popover.addClass("menu-left");
+            popover.find(".dropdown-submenu").addClass("pull-left");
         }
 
-        popover.removeClass("bottom top right left").addClass(oca_position);
         $("#PopoverOption").css({"top": top, "left": left});
 
         /* Modify the "informations" content if empty */
@@ -496,13 +519,18 @@ function DataSearchPage(backend, criterion_config, criterion_config_default, sep
         });
     }
 
-    function set_postdata() {
-        var pdata = $("#datasearch_table").getGridParam("postData") || {};
+    function set_postdata(elem, include_groupby) {
+        var pdata = $(elem).getGridParam("postData") || {};
+        if ( include_groupby ) pdata["groupby[]"] = [];
 
         $.each($("#form_search :input").serializeArray(), function(i, input) {
-            // Exclude groupby field since the goal is to reload the table
-            if ( input.name != "groupby[]" )
+            if ( input.name == "groupby[]" ) {
+                // For the listing, we exclude groupby field since the goal is to reload the table
+                if ( include_groupby ) pdata[input.name].push(input.value);
+            }
+            else {
                 pdata[input.name] = input.value;
+            }
         });
 
         return pdata;
@@ -512,8 +540,7 @@ function DataSearchPage(backend, criterion_config, criterion_config_default, sep
         CommonListing(elem, {}, {
             datatype: "json",
             url: url,
-            postData: set_postdata(),
-            colNames: columns.names,
+            postData: set_postdata(elem, false),
             colModel: columns.model,
             rowattr: function(row, data, id) {
                 if ( data._classes )
@@ -569,6 +596,23 @@ function DataSearchPage(backend, criterion_config, criterion_config_default, sep
         }, jqgrid_params);
     };
 
+    page.groupby = function(elem, columns, url, jqgrid_params) {
+        prewikka_grid(elem, {
+            datatype: "json",
+            colModel: columns.model,
+            url: url,
+            rowNum: jqgrid_params.limit,
+            pager: true,
+            pgtext: "Page {0} of unknown",
+            postData: set_postdata(elem, true),
+            loadComplete: function() {
+                _resizeGrid($(elem));
+            },
+            loadError: null,  // This prevents an error row from appearing in the grid
+            cmTemplate: $.jgrid.cmTemplate.html,
+        });
+    };
+
     /* Custom event to update datasearch */
     $("#main").on("datasearch:update", function() {
         prewikka_save_parameters($("#form_search").serializeArray());
@@ -616,6 +660,10 @@ function DataSearchPage(backend, criterion_config, criterion_config_default, sep
         $("#form_search :input[name=condensed]").val($(this).prop("checked") ? 1 : 0);
     }).change();
 
+    $("#view-config-expert").change(function() {
+        $("#datasearch_table").toggleClass("table-expert", $(this).prop("checked"));
+    }).change();
+
     $("#datasearch_table").parents(".row").change("change", function () {
         $("#main .footer-buttons .btn.needone").prop("disabled", $("#datasearch_table input.cbox:checked").length == 0);
     }).change();
@@ -653,6 +701,7 @@ function DataSearchPage(backend, criterion_config, criterion_config_default, sep
     $("#timeline").on('shown.bs.collapse hidden.bs.collapse', function() {
         var shown = $("#timeline").hasClass("collapse in");
 
+        $(".timeline-toggle > i").toggleClass("fa-minus", shown).toggleClass("fa-plus", !shown);
         $("#timeline input").attr("value", (shown) ? "1" : "0");
         prewikka_update_parameters($("#form_search :input:not(.mainmenu)").serializeArray());
 
@@ -676,11 +725,16 @@ function DataSearchPage(backend, criterion_config, criterion_config_default, sep
             hide_popover();
     });
 
-    $("#PopoverOption .dropdown-submenu").hover(function handlerIn() {
+    $("#PopoverOption .dropdown-submenu > a").hover(function() {
+        $(".dropdown-menu.panel").css({"display": ""});
         if ( $(this).offset().top + 500 > window.innerHeight )
-            $(this).find(".dropdown-menu").css({"top": "unset", "bottom": 0});
-    }, function handlerOut() {
-        $(this).find(".dropdown-menu").css({"top": "", "bottom": ""});
+            $(this).siblings(".dropdown-menu").css({"top": "unset", "bottom": 0});
+        else
+            $(this).siblings(".dropdown-menu").css({"top": "", "bottom": ""});
+    });
+
+    $("#PopoverOption").on("click", ".dropdown-menu.panel", function() {
+        $(this).css({"display": "block"});
     });
 
     var window_width = $(window).width();
