@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2019 CS-SI. All Rights Reserved.
+# Copyright (C) 2004-2020 CS GROUP - France. All Rights Reserved.
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
 # This file is part of the Prewikka program.
@@ -24,12 +24,12 @@ import itertools
 import re
 import string
 import struct
-import urllib
 
 import pkg_resources
 from prewikka import hookmanager, localization, resolve, resource, template, utils, view
 from prewikka.dataprovider import Criterion
 from prewikka.utils import html
+from prewikka.utils.html.helpers import POPOVER_HTML
 
 
 def getUriCriteria(ptype, analyzerid, messageid):
@@ -40,6 +40,13 @@ def getUriCriteria(ptype, analyzerid, messageid):
     return criteria + Criterion("%s.messageid" % ptype, "=", messageid)
 
 
+def quote(string, **kwargs):
+    if string is None:
+        return string
+
+    return utils.url.quote(string.encode("utf-8"), **kwargs)
+
+
 class Table(object):
     def __init__(self):
         self._current_table = None
@@ -515,7 +522,13 @@ class MessageSummary(Table, view.View):
                 links += obj
 
             if links:
-                meaning = resource.HTMLSource("<a class='popup_menu_toggle'>%s</a><span class='popup_menu'>%s</span>") % (meaning, links)
+                meaning = resource.HTMLNode("a", meaning, **{
+                    "data-toggle": "popover",
+                    "data-placement": "bottom",
+                    "data-html": "true",
+                    "data-content": '<span class="popup-menu">%s</span>' % links,
+                    "data-template": POPOVER_HTML,
+                })
 
             if meaning not in ignored:
                 self.newTableCol(index, resource.HTMLSource(meaning or "Data content"))
@@ -722,10 +735,10 @@ class AlertSummary(TcpIpOptions, MessageSummary):
 
             if env.enable_details:
                 if reference["origin"] in ("user-specific", "vendor-specific"):
-                    urlstr = "&url=" + urllib.quote(reference["url"], safe="")
+                    urlstr = "&url=" + quote(reference["url"], safe="")
                 else:
                     urlstr = ""
-                url = "%s?origin=%s&name=%s%s" % (env.reference_details_url, urllib.quote(reference["origin"]), urllib.quote(reference["name"]), urlstr)
+                url = "%s?origin=%s&name=%s%s" % (env.reference_details_url, quote(reference["origin"]), quote(reference["name"]), urlstr)
                 self.newTableCol(index, self.getUrlLink(reference["name"], url))
             else:
                 self.newTableCol(index, reference["name"])
@@ -1006,8 +1019,10 @@ class AlertSummary(TcpIpOptions, MessageSummary):
     def render(self, analyzerid=None, messageid=None):
         MessageSummary.render(self)
 
-        alert = env.dataprovider.get(getUriCriteria("alert", analyzerid, messageid))[0]["alert"]
+        message = env.dataprovider.get(getUriCriteria("alert", analyzerid, messageid))[0]
+        alert = message["alert"]
 
+        env.request.dataset["message"] = message
         env.request.dataset["sections"] = []
 
         self.beginSection(self.getSectionName(alert))
@@ -1096,8 +1111,10 @@ class HeartbeatSummary(MessageSummary):
     def render(self, analyzerid=None, messageid=None):
         MessageSummary.render(self)
 
-        heartbeat = env.dataprovider.get(getUriCriteria("heartbeat", analyzerid, messageid))[0]["heartbeat"]
+        message = env.dataprovider.get(getUriCriteria("heartbeat", analyzerid, messageid))[0]
+        heartbeat = message["heartbeat"]
 
+        env.request.dataset["message"] = message
         env.request.dataset["sections"] = []
 
         self.beginSection(_("Heartbeat"))
